# globalPlugins/customizedTextInfo.

import globalPluginHandler
import textInfos
import re
import appModuleHandler
from logHandler import log
import globalVars
import NVDAObjects
import config
from typing import (
	Optional,
	Dict,
)

def disableInSecureMode(decoratedCls):
	"""
	Decorator used to deactivate the class passed as a parameter in secure mode.
	Found in Noelia's placeMarkers add-on.
	"""
	if globalVars.appArgs.secure:
		return globalPluginHandler.GlobalPlugin
	return decoratedCls

class MixedUIATextRange (object):

	def _getTextFromUIARange(self, textRange):
		tempText = super(MixedUIATextRange, self)._getTextFromUIARange(textRange)
		# Our modified text which should have the same number of characters.
		newText = ' '.join([(re.sub("\w", " ", x) if len(x) ==2 else x) for x in tempText.split()])
		return newText

class MixedUIA (object):

	def __init__ (self, *args, **kwargs):
		super(MixedUIA, self).__init__(*args, **kwargs)
		self.InnerTextInfoClass = type(
			"CustomizedUIAInnerTextInfo",
			(MixedUIATextRange, self.InnerTextInfoClass),
			{}
		)

class MixedTextRange (object):

	def getTextWithFields(self, formatConfig: Optional[Dict] = None) -> textInfos.TextInfo.TextWithFieldsT:
		if not formatConfig:
			formatConfig = config.conf["documentFormatting"]
		items = super(MixedTextRange, self).getTextWithFields(formatConfig=formatConfig)
		for item in items:
			if isinstance(item, str):
				# We replace the text present at the position of the initial one.
				items[items.index(item)] = " ".join([(re.sub("\w", " ", x) if len (x) == 2 else x) for x in item.split()])
		return items

	def _getTextRange(self, start, end):
		tempText = super(MixedTextRange, self)._getTextRange(start, end)
		# Our modified text which should have the same number of characters.
		newText = ' '.join([(re.sub("\w", " ", x) if len(x) ==2 else x) for x in tempText.split()])
		return newText

@disableInSecureMode
class GlobalPlugin(globalPluginHandler.GlobalPlugin):

	def chooseNVDAObjectOverlayClasses (self, obj, clsList):
		super (GlobalPlugin, self).chooseNVDAObjectOverlayClasses (obj, clsList)
		appName = appModule = appModuleHandler.getAppNameFromProcessID(obj.processID, True)
		if obj.parent and (\
			obj.treeInterceptor or obj.parent.treeInterceptor
		):
			baseClass = obj.__class__
			baseTextInfo = obj.TextInfo
			baseTreeInterceptor = (
				obj.treeInterceptor.__class__ if obj.treeInterceptor else obj.parent.treeInterceptor.__class__
			)
			baseTreeInterceptorTextInfo = (
				obj.treeInterceptor.TextInfo if obj.treeInterceptor else obj.parent.treeInterceptor.TextInfo
			)
			if not baseTreeInterceptor.__name__ == "CustomizedTreeInterceptorClass":
				CustomizedTreeInterceptorClass = type(
					"CustomizedTreeInterceptorClass",
					(baseTreeInterceptor,),
					{}
				)
			if not baseTreeInterceptorTextInfo.__name__ == "CustomizedTreeInterceptorTextInfo":
				CustomizedTreeInterceptorTextInfo = type(
					"CustomizedTreeInterceptorTextInfo",
					(MixedTextRange, baseTreeInterceptorTextInfo),
					{"_get_obj": lambda self: (obj.treeInterceptor if obj.treeInterceptor else obj.parent.treeInterceptor)}
				)
			if not baseTextInfo.__name__ == "CustomizedTextInfo":
				CustomizedTextInfo = type(
					"CustomizedTextInfo",
					(MixedTextRange, baseTextInfo),
					{}
				)
			if (
				config.conf["UIA"]["allowInMSWord"] == 3
				or config.conf["UIA"]["allowInChromium"] == 2
			):
				if appName in ("WINWORD.EXE", "msedge.exe"):
					if not baseTreeInterceptorTextInfo.__name__ == "CustomizedTreeInterceptorTextInfo":
						CustomizedTreeInterceptorTextInfo = type(
							"CustomizedTreeInterceptorTextInfo",
							(baseTreeInterceptorTextInfo, MixedUIA),
							{"_get_obj": lambda self: (obj.treeInterceptor if obj.treeInterceptor else obj.parent.treeInterceptor)}
						)
					if not baseTextInfo.__name__ == "CustomizedTextInfo":
						CustomizedTextInfo = type(
							"CustomizedTextInfo",
							(MixedUIATextRange, baseTextInfo),
							{}
						)
			if obj.treeInterceptor:
				if not obj.treeInterceptor.__class__.__name__ == "CustomizedTreeInterceptorClass":
					obj.treeInterceptor.__class__ = CustomizedTreeInterceptorClass
					obj.treeInterceptorClass = CustomizedTreeInterceptorClass
				if not obj.treeInterceptor.TextInfo.__name__ == "CustomizedTreeInterceptorTextInfo":
					obj.treeInterceptor.TextInfo = CustomizedTreeInterceptorTextInfo
			if obj.parent.treeInterceptor:
				if not obj.parent.treeInterceptor.__class__.__name__ == "CustomizedTreeInterceptorClass":
					obj.parent.treeInterceptor.__class__ = CustomizedTreeInterceptorClass
					obj.parent.treeInterceptorClass = CustomizedTreeInterceptorClass
				if not obj.parent.treeInterceptor.TextInfo.__name__ == "CustomizedTreeInterceptorTextInfo":
					obj.parent.treeInterceptor.TextInfo = CustomizedTreeInterceptorTextInfo
			CustomizedWebNVDAClass = type(
				"CustomizedWebNVDAClass",
				(baseClass,),
				{"TextInfo": CustomizedTextInfo}
			)
			clsList.insert (0, CustomizedWebNVDAClass)
		if appName == "notepad.exe" and obj.windowControlID == 15\
		or appName == "notepad++.exe" and obj.windowControlID == 0:
			baseClass = obj.__class__
			baseTextInfo = obj.TextInfo
			if not baseTextInfo.__name__ == "CustomizedEditorTextInfo":
					CustomizedEditorTextInfo = type(
						"CustomizedEditorTextInfo", 
						(MixedTextRange, NVDAObjects.window.edit.EditTextInfo),
						{}
					) if appName == "notepad.exe" else type(
						"CustomizedEditorTextInfo",
						(MixedTextRange, NVDAObjects.window.scintilla.ScintillaTextInfo),
						{}
					)
					if not baseClass.__name__ == "CustomizedEditorClass":
						CustomizedEditorClass = type(
						"CustomizedEditorClass",
						(baseClass,),
						{"TextInfo": CustomizedEditorTextInfo}
					)
			clsList.insert (0, CustomizedEditorClass)
